from flask import Flask, jsonify
from .config import LibConfig
from .registration import Registration
import threading

name = "Libras"


class RepsonseAdvice(object):

    @staticmethod
    def before_write(rv):
        return rv


class Libras(Flask):
    config_class = LibConfig

    repsonse_advice = RepsonseAdvice

    def property(self, key: str):
        return self.config.property(key)

    def make_response(self, rv, types: iter = (dict, list, set)):
        if isinstance(rv, types):
            rv = jsonify(self.repsonse_advice.before_write(rv))
        return super(Libras, self).make_response(rv)


class Application(object):
    _instance_lock = threading.Lock()

    def __init__(self, packages: str = None, profile: str = None):
        self.app = Libras(__name__)
        self.initialization(packages=packages, profile=profile)

    def __new__(cls, *args, **kwargs):
        if not hasattr(Application, "_instance"):
            with Application._instance_lock:
                if not hasattr(Application, "_instance"):
                    Application._instance = object.__new__(cls)
        return Application._instance

    def initialization(self, packages: str = None, profile: str = None):
        self.app.config.from_yaml(profile)
        self.app.config['JSON_AS_ASCII'] = False
        self.app.config.update(RESTFUL_JSON=dict(ensure_ascii=False))
        self.app.config['scan_packages'] = packages
        Registration.register(self.app)

    def run(self, host=None, port=9003):
        self.app.run(host=host, port=port, debug=True)
