from flask import jsonify, g, request, Response
from .exception import FlaskException
import time
import json


class Registration(object):

    @staticmethod
    def global_exception(app):
        @app.errorhandler(FlaskException)
        def handle_invalid_usage(error):
            resp = jsonify(error.__dict__)
            resp.headers['Content-Type'] = 'application/json;charset=UTF-8'
            resp.status_code = error.status_code
            return resp

        @app.errorhandler(Exception)
        def handle_default_error(error):
            from werkzeug.http import HTTP_STATUS_CODES
            resp = Response()
            code = 400
            if hasattr(error, 'code'):
                code = error.code
            default_data = {
                'message': '网络不给力',
                'error_message': getattr(error, 'description', HTTP_STATUS_CODES.get(code)),
                'code': code
            }
            resp.set_data(json.dumps(default_data))
            resp.headers['Content-Type'] = 'application/json;charset=UTF-8'
            resp.status_code = code
            return resp

    @staticmethod
    def register_before_request(app):
        @app.before_request
        def request_cost_time():
            g.request_start_time = time.time()
            g.request_time = lambda: "%.5f" % (time.time() - g.request_start_time)

    @staticmethod
    def register_after_request(app):
        @app.after_request
        def log_response(resp):
            log_config = app.property['log']
            message = '[%s] - [%s] -> [%s] costs:%.3f ms' % (
                request.remote_addr,
                request.method,
                request.path,
                float(g.request_time()) * 1000
            )
            if log_config['level'] == 'INFO':
                app.logger.info(message)
            elif log_config['level'] == 'DEBUG':
                req_body = '{}'
                try:
                    req_body = request.get_json() if request.get_json() else {}
                except:
                    pass
                message += " data:{\n\tparam: %s, \n\tbody: %s\n} " % (
                    json.dumps(request.args, ensure_ascii=False),
                    req_body
                )
                app.logger.debug(message)
            return resp

    @staticmethod
    def register_blueprints(app):
        from Libras.mapping import import_modules
        import_modules(app)

    @staticmethod
    def register_db(app):
        pass
        # if app.prop("SQLALCHEMY_DATABASE_URI"):
        #     pass
            # from .db.mysql import db
            # db.init_app(app)

    @staticmethod
    def register(app):
        Registration.global_exception(app)
        Registration.register_before_request(app)
        Registration.register_after_request(app)
        Registration.register_blueprints(app)
        Registration.register_db(app)
